require "rails/generators"

module GraphqlAuthorizer
  module Generators
    # Add GraphQLAuthorizer to a Rails app with `rails g graphql_authorizer:install`.
    #
    # Setup a initializer file Rack Attack and GraphQLAuhtorizer
    #
    # ```
    # - app/
    # - config/
    #   - initializers/
    #     - rack_attack.rb
    #     - graphql_authorizer.rb
    # ```
    class InstallGenerator < Rails::Generators::Base
      desc "Copy Rack Attack Template into App's Initializer"
      source_root File.expand_path('templates', __dir__)

      def copy_graphql_authorizer
        file = "graphql_authorizer.rb"
        template(file, "config/initializers/#{file}")
      end

      def inject_rack_attack_in_gemfile
        gem("rack-attack")
        print "\nGemfile has been modified, make sure you `bundle install\n`"
      end

      def copy_rack_attack
        file = "rack_attack.rb"
        template(file, "config/initializers/#{file}")
      end

      private

      def gem(*args)
        print "\nAdding Gem into Gemfile\n"
        super(*args)
      end
    end
  end
end
