require "active_support/core_ext/array/conversions"
require "errors/configuration"
require_relative "validator/signature"
require_relative "validator/timestamp"

module GraphQLAuthorizer
  class Request
    attr_accessor :errors

    include GraphQLAuthorizer::Validator::Signature
    include GraphQLAuthorizer::Validator::Timestamp

    def initialize(sig:, timestamp:)
      @sig = sig.to_s
      @timestamp = timestamp.to_s
      @errors = []
      config = GraphQLAuthorizer.configuration
      raise_missing_configuration unless config
    end

    def valid?
      valid_sig = request_signature_is_valid?(sig: @sig, timestamp: @timestamp)
      @errors << invalid_signature unless valid_sig
      valid_timestamp = request_timestamp_is_valid?(timestamp: @timestamp)
      @errors << invalid_timestamp unless valid_timestamp
      valid_sig && valid_timestamp
    end

    private

    def invalid_signature
      "Invalid Signature!"
    end

    def invalid_timestamp
      "Invalid Timestamp!"
    end

    def raise_missing_configuration
      raise GraphQLAuthorizer::Errors::Configuration,
            "Missing Configuration block in initializers #{readme_link}"
    end

    def readme_link
      "https://bitbucket.org/gorated/graphql-authorizer/src/development/README.md"
    end
  end
end
